//@author: vux
//@help: template for standard shaders
//@tags: template
//@credits: 


	float4x4 tVP : VIEWPROJECTION;	
	float4x4 tW : WORLD;

struct VS_IN
{
	float4 pos : POSITION;
	float4 uv : TEXCOORD0;

};

struct VS_OUT
{
    float4 pos: SV_POSITION;
    float4 uv: TEXCOORD0;
};

VS_OUT VS(VS_IN input)
{
    VS_OUT output;
    output.pos  = mul(input.pos,tW);
    output.uv= input.uv;
    return output;
}


float box (float3 p) {return max(p.x,max(p.y,p.z));}
float sdTorus( float3 p, float2 t )
{
  float2 q = float2(length(p.xz)-t.x,p.y);
  return length(q)-t.y;
}


float2 r2d(float2 x,float a){a*=acos(-1)*2;return float2(cos(a)*x.x+sin(a)*x.y,cos(a)*x.y-sin(a)*x.x);}

float test_value;

float f2 (float3 p)
{
	float2 cellID = 1 - frac (p.xz)*2;
	p.xz = 1 - frac (p.xz)*2;
	float d = box(abs(p) - float3 (0.2,length (cellID),0.2));
	return d;
}


float f (float3 p)
{
	//p.xz = 1 - frac (p.xz)*2;
	float d = 99999;
	
	float3 p1 =p;
	
	float dtorus = sdTorus (p,float2 (1 , 0.5));
	
	p.z = abs (p.z);
	p.x = (1 - abs (p.x)*4)/4;
	p.y = (0 + abs (p.y)*4)/4;
	p.xz = r2d (p.xz, sin (length (p.xz))*0.5);
	d = length(p.yz) - 0.25 + test_value * length (p.xz);
	
	float dbox = box(abs(p + float3 (1,0,1)) -1 + test_value * length (p)); 
	
	d = min(d,dbox);
	d = max(d,-dtorus);
	
	return d;//f2(p1); 
}

float map (float3 p)
{
	return min (f(p), p.y + 0.5);
}

float raymarch (in float3 ro, in float3 rd, inout float3 p)
{
	float t = 0.0;
	for (int i = 0 ; i < 128 ; i++)
	{
		p = ro + rd*t;
		float d = map (p);
		t += d * 0.5;
	}
	return t;
}

float4x4 tVI:VIEWINVERSE;
float4x4 tPI:PROJECTIONINVERSE;
float3 UVtoEYE(float2 UV){return normalize(mul(float4(mul(float4((UV.xy*2-1)*float2(1,-1),0,1),tPI).xy,1,0),tVI).xyz);}

float3 calcNormal( in float3 pos )
{
	float3 eps = float3( 0.0001, 0.0, 0.0 );
	float3 nor = float3(
	    map(pos+eps.xyy) - map(pos-eps.xyy),
	    map(pos+eps.yxy) - map(pos-eps.yxy),
	    map(pos+eps.yyx) - map(pos-eps.yyx) );
	return normalize(nor);
}

struct PS_OUT
{
	float4 color : SV_Target0;
	float depth : SV_Depth;
};


PS_OUT PS(VS_OUT input)
{
    PS_OUT Out;
	float3 p;
	
	//float3 ro = float3 (0,0,-3);
	//float3 rd = float3 (2 * input.uv.xy - 1,1);
	
	float3 ro = tVI[3].xyz;
	float3 rd = UVtoEYE(input.uv.xy);
	
	float d = raymarch (ro, rd, p);
	float3 normal = calcNormal(p);
	
	float fog = 1/(1 + d * d *0.25);
	float4 col = float4 (normal * fog,1) ;
		
	float not_grid = f(p);
	if (not_grid > 0.01)
	{
		col.rgb *= saturate(abs(frac(not_grid*10)*2-1)*10);
	}
	
	float4 pos = mul (float4 (p.xyz,1),tVP);
	
	
	
	if (abs(map(p)) > 0.01) discard;
	
	Out.color = col;
	Out.depth = pos.z/pos.w;
	
    return Out;
}





technique10 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




